@@ -189,7 +189,7 @@ class Video(object):
                 # translate path for windows
                 # if path is smb path in credential format for kodi and maybe linux \\username:password@mediaserver\foo,
                 # translate it to mediaserver/foo
-                return pathlib.Path(self.media_source["Path"])
+                return str(pathlib.Path(self.media_source["Path"]))
             else:
                 # If there's no uri scheme, check if the file exixsts because it might not be mounted
                 if os.path.isfile(self.media_source["Path"]):
@@ -257,6 +257,7 @@ class Video(object):
         return [
             {"start": item["StartPositionTicks"] / 10000000, "name": item["Name"]}
             for item in self.item.get("Chapters", [])
+            if item.get("ImageTag")
         ]
 
     def get_chapter_images(self, max_width=400, quality=90):
@@ -269,9 +270,16 @@ class Video(object):
             )
             yield data.getvalue()
 
+    def get_hls_tile_images(self, width, count):
+        for i in range(1, count + 1):
+            data = BytesIO()
+            self.client.jellyfin._get_stream(
+                f"Trickplay/{self.media_source['Id']}/{width}/{i}.jpg", data
+            )
+            yield data.getvalue()
+
     def get_bif(self, prefer_width=320):
         # requires JellyScrub plugin
-        data = BytesIO()
         manifest = self.client.jellyfin._get(
             f"Trickplay/{self.media_source['Id']}/GetManifest"
         )
@@ -280,19 +288,27 @@ class Video(object):
             and manifest.get("WidthResolutions") is not None
             and len(manifest["WidthResolutions"]) > 0
         ):
+            available_widths = manifest["WidthResolutions"]
+            if type(manifest["WidthResolutions"]) is dict:
+                available_widths = [int(x) for x in manifest["WidthResolutions"].keys()]
+
             if prefer_width is not None:
-                width = min(
-                    manifest["WidthResolutions"], key=lambda x: abs(x - prefer_width)
-                )
+                width = min(available_widths, key=lambda x: abs(x - prefer_width))
             else:
-                width = manifest["WidthResolutions"][-1]
-            self.client.jellyfin._get_stream(
-                f"Trickplay/{self.media_source['Id']}/{width}/GetBIF", data
-            )
+                width = max(available_widths)
+
+            if type(manifest["WidthResolutions"]) is dict:
+                return manifest["WidthResolutions"][str(width)]
+            else:
+                data = BytesIO()
+                self.client.jellyfin._get_stream(
+                    f"Trickplay/{self.media_source['Id']}/{width}/GetBIF", data
+                )
+
+                data.seek(0)
+                return data
         else:
             return None
-        data.seek(0)
-        return data
 
     def get_playback_url(
         self,
