@@ -176,30 +176,40 @@ class PlayerManager(object):
                 self.trickplay = TrickPlay(self)
                 self.trickplay.start()
 
-                if settings.thumbnail_custom_script:
-                    scripts.append(settings.thumbnail_custom_script)
-                else:
-                    scripts.append(get_resource("trickplay.lua"))
+                scripts.append(get_resource("thumbfast.lua"))
+                if settings.thumbnail_osc_builtin:
+                    scripts.append(get_resource("trickplay-osc.lua"))
+
             except Exception:
                 log.error("Could not enable trickplay.", exc_info=True)
 
             mpv_options["osc"] = False
 
+        # ensure standard mpv configuration directories and files exist
+        conffile.get_dir(APP_NAME, "scripts")
+        conffile.get_dir(APP_NAME, "fonts")
+        conffile.get(APP_NAME, "input.conf", True)
+        conffile.get(APP_NAME, "mpv.conf", True)
+
         if scripts:
-            mpv_options["scripts"] = (
-                ";" if sys.platform.startswith("win32") else ":"
-            ).join(scripts)
+            if settings.mpv_ext:
+                mpv_options["script"] = scripts
+            else:
+                mpv_options["scripts"] = (
+                    ";" if sys.platform.startswith("win32") else ":"
+                ).join(scripts)
 
         if not (settings.mpv_ext and settings.mpv_ext_no_ovr):
-            mpv_options["include"] = conffile.get(APP_NAME, "mpv.conf", True)
-            mpv_options["input_conf"] = conffile.get(APP_NAME, "input.conf", True)
+            mpv_options["config"] = True
+            mpv_options["config_dir"] = conffile.confdir(APP_NAME)
+
         self._player = mpv.MPV(
             input_default_bindings=True,
             input_vo_keyboard=True,
             input_media_keys=settings.media_keys,
             log_handler=mpv_log_handler,
             loglevel=settings.mpv_log_level,
-            **mpv_options
+            **mpv_options,
         )
         self.menu = OSDMenu(self, self._player)
         self.syncplay = SyncPlayManager(self)
@@ -1054,7 +1064,7 @@ class PlayerManager(object):
     def enable_osc(self, enabled: bool):
         if settings.thumbnail_enable and self.trickplay:
             self.script_message(
-                "osc-visibility", "always" if enabled else "never", "False"
+                "osc-visibility", "auto" if enabled else "never", "False"
             )
         else:
             if hasattr(self._player, "osc"):
